ENTRY_POINT	equ 64400

; ==============================
; #  STAR WARS Style Scroller  #
; ==============================
;
; Jason 'Joefish' Railton, 18 Aug 2023
;
; Free for any use. No warranty supplied.
; This can probably be optimised a lot.
;
; To reset, POKE the start address +1 with 0
; To use, print 16 characters of text AT 0,0
; Then call this routine 16 times.
; Then PRINT another line of text, 16 characters AT 0,0
; and repeat.
;
; Attributes are not affected, so your PRINTed text
; should be hidden by using the same INK and PAPER.
; Set the attributes for the rest of the screen to
; make the scroller visible.
;
;Compiled code is just under 800 bytes long

org ENTRY_POINT
do_scroll
scroll_step EQU $+1
	ld a,0

get_scroll_size
	ld a,(scroll_step)
	and SCROLL_SIZE_AND
	add a,scroll_size_table
	ld l,a
	ld h,scroll_size_table/256

	ld a,LINES_TABLE_LENGTH
	sub (hl)
	and 254
	ld b,0
	ld c,a
	ld iy,lines_table
	add iy,bc
	add iy,bc
	ld ix,shift_table
	srl c
	add ix,bc
			
	ld e,(iy)
	inc iy
	ld d,(iy)
	inc iy
	
	ld a,(hl)
	and 254
	dec a
		
scroll_lines_loop
	ex af,af'
	
	ex de,hl
	ld e,(iy)
	inc iy
	ld d,(iy)
	inc iy
	ex de,hl
	
	ld (scroll_lines_repeat_a+1),hl
	
	ld b,(ix)
	and a
squeeze_lines_right_loop
	ld a,(hl)
	rr a
	ld (de),a
	inc l
	inc e
	djnz squeeze_lines_right_loop
	
	ld a,32
	sub (ix)
	ld b,0
	ld c,a
	ldir

scroll_lines_repeat_a	
	ld de,0000
	
	ex af,af'
	dec a
	jp z, scroll_lines_done
	ex af,af'
	
	ex de,hl
	ld e,(iy)
	inc iy
	ld d,(iy)
	inc iy
	ex de,hl
	
	ld (scroll_lines_repeat_b+1),hl
	
	ld bc,31
	add hl,bc
	ex de,hl
	add hl,bc
	ex de,hl
	
	ld b,(ix)
	and a
squeeze_lines_left_loop
	ld a,(hl)
	rl a
	ld (de),a
	dec l
	dec e
	djnz squeeze_lines_left_loop
	
	ld a,32
	sub (ix)
	ld b,0
	ld c,a
	lddr
	
scroll_lines_repeat_b
	ld de,0000
	
	ex af,af'
	inc ix
	dec a
	jp nz, scroll_lines_loop
scroll_lines_done
	
and_fade
	ld de,16384+(32*4)+9
	ld hl,and_table
	ld a,4
and_loop_a
	ex af,af'
	ld c,(hl)
	ld b,14
and_loop_b
	ld a,(de)
	and c
	ld (de),a
	inc de
	djnz and_loop_b
	inc d
	ld e,9+(32*4)
	inc hl
	ex af,af'
	dec a
	jp nz,and_loop_a
		
	ld a,(scroll_step)
	inc a
	ld (scroll_step),a
and_fade_done	
	
text_expand
	ld de,16384
	ld hl,16384 + 6144 - 32
	ld b,16
text_expand_loop
	ld a,(de)
	
	ld c,0
	rrca
	rr c
	sra c
	  rrca
	  rr c
	  sra c
	rrca
	rr c
	sra c
	  rra
	  rr c
	  sra c
	  
	inc l
	ld (hl),c
	dec l
	
	rrca
	rr c
	sra c
	  rrca
	  rr c
	  sra c
	rrca
	rr c
	sra c
	  rrca
	  rr c
	  sra c
	ld (hl),c
	
	inc l
	inc l
	
	inc e
	djnz text_expand_loop
text_expand_done
	
	ld a,(scroll_step)
	and 1
	jp z,text_up_done
	
text_up
	ld hl,16384 + 256
	ld de,16384
	ld a,7
text_up_loop

	REPT 16
		ldi
	ENDM
	
	inc h
	inc d
	ld l,0
	ld e,0
	
	dec a
	jp nz,text_up_loop
text_up_done

	ret

and_table
	db %10101010
	db %01010101
	db %10111011
	db %11011101
	
scroll_size_table
	db 160, 80, 120, 140,  100, 160, 60, 150
	SCROLL_SIZE_AND		EQU 7
	
shift_table
	;12
	db	1,	1,	1,	1,	1,	1,	1,	1
	;11
	db	1,	1,	1,	1,	1,	1,	1,	1
	;10
	db	16,	16,	16,	16,	11,	13,	15,	12
	;9
	db	12,	14,	10,	15,	13,	11,	14,	12	
	;8
	db	9,	13,	11,	14,	10,	12,	15,	10
	;7
	db	10,	14,	15,	12,	8,	13,	9, 11	
	;6
	db	9,	14,	7,	11,	13,	10,	8,	12
	;5
	db	7,	13,	5,	8,	11,	6,	9,	15
	;4
	db	5,	15,	9,	11,	4,	12,	7,	13
	;3
	db	8,	14,	5,	7,	13,	3,	9,	12
	;2
	db	6,	11,	3,	9,	15,	2,	14,	7
	;1
	db	9,	12,	4,	14,	2,	7,	13,	5
		
lines_table
	REPT 3, screen_third
		REPT 8, char_row
			REPT 8, pixel_row
				dw 16384 + (2048 * screen_third) + (32 * char_row) + (256 * pixel_row)
			ENDM
		ENDM
	ENDM
	LINES_TABLE_LENGTH EQU ($-lines_table)/2
	
end ENTRY_POINT
